# -*- coding: utf-8 -*-

from ScoutSuite.providers.gcp.configs.base import GCPBaseConfig

from googleapiclient.errors import HttpError
import json
from opinel.utils.console import printException, printError

class ComputeEngineConfig(GCPBaseConfig):
    targets = (
        ('instances', 'Instances', 'list', {'project': 'project_placeholder', 'zone': 'zone_placeholder'}, False),
        ('snapshots', 'Snapshots', 'list', {'project': 'project_placeholder'}, False),
        ('networks', 'Networks', 'list', {'project': 'project_placeholder'}, False),
        # ('subnetworks', 'Subnetworks', 'list', {'project': 'project_placeholder', 'region': 'region_placeholder'}, False),
        ('firewalls', 'Firewalls', 'list', {'project': 'project_placeholder'}, False),
    )

    def __init__(self, thread_config):
        self.library_type = 'api_client_library'
        self.zones = None

        self.instances = {}
        self.instances_count = 0
        self.snapshots = {}
        self.snapshots_count = 0
        self.networks = {}
        self.networks_count = 0
        self.firewalls = {}
        self.firewalls_count = 0

        # TODO figure out why GCP returns errors when running with more then 1 thread (multithreading)
        super(ComputeEngineConfig, self).__init__(thread_config=1)

    def get_zones(self, client, project):
        try:
            if self.zones:
                return self.zones
            else:
                zones_list = []
                zones = client.zones().list(project=project).execute()['items']
                for zone in zones:
                    zones_list.append(zone['name'])
                self.zones = zones_list
                return zones_list

        except HttpError as e:
            raise e

        except Exception as e:
            printException(e)
            return None

    def parse_instances(self, instance, params):
        instance_dict = {}
        instance_dict['id'] = self.get_non_provider_id(instance['name'])
        instance_dict['project_id'] = instance['selfLink'].split('/')[-5]
        instance_dict['name'] = instance['name']
        instance_dict['description'] = instance['description'] if 'description' in instance else 'N/A'
        instance_dict['creation_timestamp'] = instance['creationTimestamp']
        instance_dict['zone'] = instance['zone'].split('/')[-1]
        instance_dict['tags'] = instance['tags']
        instance_dict['status'] = instance['status']
        instance_dict['zone_url_'] = instance['zone']
        instance_dict['network_interfaces'] = instance['networkInterfaces']
        instance_dict['service_accounts'] = instance['serviceAccounts']
        instance_dict['deletion_protection'] = 'Enabled' if instance['deletionProtection'] else 'Disabled'

        instance_dict['disks'] = {}
        if 'disks' in instance:
            for disk in instance['disks']:
                instance_dict['disks'][self.get_non_provider_id(disk['deviceName'])] = {
                    'type': disk['type'],
                    'mode': disk['mode'],
                    'source_url': disk['source'],
                    'source_device_name': disk['deviceName'],
                    'bootable': disk['boot']
                    }

        self.instances[instance_dict['id']] = instance_dict

    def parse_snapshots(self, snapshot, params):
        snapshot_dict = {}
        snapshot_dict['id'] = snapshot['id']
        snapshot_dict['name'] = snapshot['name']
        snapshot_dict['description'] = snapshot['description'] if snapshot['description'] else 'N/A'
        snapshot_dict['creation_timestamp'] = snapshot['creationTimestamp']
        snapshot_dict['status'] = snapshot['status']
        snapshot_dict['source_disk_id'] = snapshot['sourceDiskId']
        snapshot_dict['source_disk_url'] = snapshot['sourceDisk']
        self.snapshots[snapshot_dict['id']] = snapshot_dict

    def parse_networks(self, network, params):
        network_dict = {}
        network_dict['id'] = network['id']
        network_dict['name'] = network['name']
        network_dict['description'] = network['description'] if 'description' in network and network['description'] else 'N/A'
        network_dict['creation_timestamp'] = network['creationTimestamp']
        network_dict['network_url'] = network['selfLink']
        network_dict['subnetwork_urls'] = network['subnetworks']
        network_dict['auto_subnet'] = network['autoCreateSubnetworks']
        network_dict['routing_config'] = network['routingConfig']
        self.networks[network_dict['id']] = network_dict

    def parse_subnetworks(self, subnetwork, params):
        subnetwork_dict = {}
        subnetwork_dict['id'] = subnetwork['id']
        self.networks[subnetwork_dict['id']] = subnetwork_dict

    def parse_firewalls(self, firewall, params):
        firewall_dict = {}
        firewall_dict['id'] = firewall['id']
        firewall_dict['name'] = firewall['name']
        firewall_dict['description'] = firewall['description'] if 'description' in firewall else 'N/A'
        firewall_dict['creation_timestamp'] = firewall['creationTimestamp']
        firewall_dict['network'] = firewall['network'].split('/')[-1]
        firewall_dict['network_url'] = firewall['network']
        firewall_dict['priority'] = firewall['priority']
        firewall_dict['source_ranges'] = firewall['sourceRanges'] if 'sourceRanges' in firewall else []
        firewall_dict['source_tags'] = firewall['sourceTags'] if 'sourceTags' in firewall else []
        firewall_dict['target_tags'] = firewall['targetTags'] if 'targetTags' in firewall else []
        firewall_dict['direction'] = firewall['direction']
        firewall_dict['disabled'] = firewall['disabled']

        # Parse FW rules
        for direction in ['allowed', 'denied']:
            direction_string = '%s_traffic' % direction
            firewall_dict[direction_string] = {
                'tcp': [],
                'udp': [],
                'icmp': []
            }
            if direction in firewall:
                firewall_dict['action'] = direction
                for rule in firewall[direction]:
                    if rule['IPProtocol'] not in firewall_dict[direction_string]:
                        firewall_dict[direction_string][rule['IPProtocol']] = ['*']
                    else:
                        if rule['IPProtocol'] == 'all':
                            for protocol in firewall_dict[direction_string]:
                                firewall_dict[direction_string][protocol] = ['0-65535']
                            break
                        else:
                            if firewall_dict[direction_string][rule['IPProtocol']] != ['0-65535']:
                                if 'ports' in rule:
                                    firewall_dict[direction_string][rule['IPProtocol']] += rule['ports']
                                else:
                                    firewall_dict[direction_string][rule['IPProtocol']] = ['0-65535']

        self.firewalls[firewall_dict['id']] = firewall_dict



# -*- coding: utf-8 -*-

from opinel.utils.console import printException, printInfo

from azure.mgmt.storage import StorageManagementClient

def azure_connect_service(service, credentials, region_name=None):

    try:
        if service == 'storageaccounts':
            return StorageManagementClient(credentials.credentials, credentials.subscription_id)

        else:
            printException('Service %s not supported' % service)
            return None

    except Exception as e:
        printException(e)
        return None



# -*- coding: utf-8 -*-

from ScoutSuite.providers.azure.configs.base import AzureBaseConfig

from opinel.utils.console import printError, printException, printInfo


class StorageAccountsConfig(AzureBaseConfig):
    targets = (
        ('storage_accounts', 'Storage Accounts', 'list', {}, False),
    )

    def __init__(self, thread_config):

        self.storage_accounts = {}
        self.storage_accounts_count = 0

        super(StorageAccountsConfig, self).__init__(thread_config)

    def parse_storage_accounts(self, storage_account, params):

        storage_account_dict = {}

        storage_account_dict['id'] = self.get_non_provider_id(storage_account.id)
        storage_account_dict['name'] = storage_account.name
        storage_account_dict['https_traffic_enabled'] = 'Enabled' if storage_account.enable_https_traffic_only else 'Disabled'

        self.storage_accounts[storage_account_dict['id']] = storage_account_dict




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import copy
import json
import os
import sys
import webbrowser

try:
    from opinel.utils.aws import get_aws_account_id, get_partition_name
    from opinel.utils.console import configPrintException, printInfo, printDebug
    from opinel.utils.credentials import read_creds
    from opinel.utils.globals import check_requirements
    from opinel.utils.profiles import AWSProfiles
except Exception as e:
    print('Error: Scout2 depends on the opinel package. Install all the requirements with the following command:')
    print('  $ pip install -r requirements.txt')
    print(e)
    sys.exit(42)

from ScoutSuite import AWSCONFIG
from ScoutSuite.cli_parser import ScoutSuiteArgumentParser
# from ScoutSuite.configs.scout2 import Scout2Config
from ScoutSuite.output.html import Scout2Report
from ScoutSuite.core.exceptions import RuleExceptions
from ScoutSuite.core.ruleset import Ruleset
from ScoutSuite.core.processingengine import ProcessingEngine
from ScoutSuite.providers import get_provider


def main(passed_args=None):
    """
    Main method that runs a scan

    :return:
    """

    # Parse arguments
    parser = ScoutSuiteArgumentParser()

    if passed_args:
        args = parser.parse_args(passed_args)
    else:
        args = parser.parse_args()

    # Configure the debug level
    configPrintException(args.debug)

    # Create a cloud provider object
    cloud_provider = get_provider(provider=args.provider,
                                  profile=args.profile[0],
                                  project_id=args.project_id,
                                  folder_id=args.folder_id,
                                  organization_id=args.organization_id,
                                  report_dir=args.report_dir,
                                  timestamp=args.timestamp,
                                  services=args.services,
                                  skipped_services=args.skipped_services,
                                  thread_config=args.thread_config)

    #FIXME this shouldn't be done here
    if cloud_provider.provider_code == 'aws':
        report_file_name = 'aws-%s' % args.profile[0]
    if cloud_provider.provider_code == 'gcp':
        if args.project_id:
            report_file_name = 'gcp-%s' % args.project_id
        elif args.organization_id:
            report_file_name = 'gcp-%s' % args.organization_id
        elif args.folder_id:
            report_file_name = 'gcp-%s' % args.folder_id
        else:
            report_file_name = 'gcp'
    if cloud_provider.provider_code == 'azure':
        report_file_name = 'azure'

    # Create a new report
    report = Scout2Report(args.provider, report_file_name, args.report_dir, args.timestamp)

    # Complete run, including pulling data from provider
    if not args.fetch_local:
        # Authenticate to the cloud provider
        authenticated = cloud_provider.authenticate(profile=args.profile[0],
                                                    csv_credentials=args.csv_credentials,
                                                    mfa_serial=args.mfa_serial,
                                                    mfa_code=args.mfa_code,
                                                    key_file=args.key_file,
                                                    user_account=args.user_account,
                                                    service_account=args.service_account)
        if not authenticated:
            return 42

        # Fetch data from provider APIs
        try:
            cloud_provider.fetch(regions=args.regions)
        except KeyboardInterrupt:
            printInfo('\nCancelled by user')
            return 130

        # Update means we reload the whole config and overwrite part of it
        if args.update:
            current_run_services = copy.deepcopy(cloud_provider.services)
            last_run_dict = report.jsrw.load_from_file(AWSCONFIG)
            cloud_provider.services = last_run_dict['services']
            for service in cloud_provider.service_list:
                cloud_provider.services[service] = current_run_services[service]

    # Partial run, using pre-pulled data
    else:
        # Reload to flatten everything into a python dictionary
        last_run_dict = report.jsrw.load_from_file(AWSCONFIG)
        for key in last_run_dict:
            setattr(cloud_provider, key, last_run_dict[key])

    # Pre processing
    cloud_provider.preprocessing(args.ip_ranges, args.ip_ranges_name_key)

    # # Analyze config
    finding_rules = Ruleset(environment_name=args.profile[0],
                            cloud_provider=args.provider,
                            filename=args.ruleset,
                            ip_ranges=args.ip_ranges,
                            aws_account_id=cloud_provider.aws_account_id)
    processing_engine = ProcessingEngine(finding_rules)
    processing_engine.run(cloud_provider)

    # Create display filters
    filter_rules = Ruleset(cloud_provider=args.provider,
                           filename='filters.json',
                           rule_type='filters',
                           aws_account_id=cloud_provider.aws_account_id)
    processing_engine = ProcessingEngine(filter_rules)
    processing_engine.run(cloud_provider)

    # Handle exceptions
    try:
        exceptions = RuleExceptions(args.profile[0], args.exceptions[0])
        exceptions.process(cloud_provider)
        exceptions = exceptions.exceptions
    except Exception as e:
        printDebug('Warning, failed to load exceptions. The file may not exist or may have an invalid format.')
        exceptions = {}

    # Finalize
    cloud_provider.postprocessing(report.current_time, finding_rules)

    # TODO this is AWS-specific - move to postprocessing?
    # Get organization data if it exists
    try:
        profile = AWSProfiles.get(args.profile[0])[0]
        if 'source_profile' in profile.attributes:
            organization_info_file = os.path.join(os.path.expanduser('~/.aws/recipes/%s/organization.json' %
                                                                     profile.attributes['source_profile']))
            if os.path.isfile(organization_info_file):
                with open(organization_info_file, 'rt') as f:
                    org = {}
                    accounts = json.load(f)
                    for account in accounts:
                        account_id = account.pop('Id')
                        org[account_id] = account
                    setattr(cloud_provider, 'organization', org)
    except Exception as e:
        pass

    # Save config and create HTML report
    html_report_path = report.save(cloud_provider, exceptions, args.force_write, args.debug)

    # Open the report by default
    if not args.no_browser:
        printInfo('Opening the HTML report...')
        url = 'file://%s' % os.path.abspath(html_report_path)
        webbrowser.open(url, new=2)

    return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import webbrowser

try:
    from opinel.utils.console import configPrintException, printInfo
    from opinel.utils.globals import check_requirements
except Exception as e:
    print('Error: Scout2 depends on the opinel package. Install all the requirements with the following command:')
    print('  $ pip install -r requirements.txt')
    print(e)
    sys.exit(42)

from ScoutSuite.cli_parser import RulesArgumentParser
# from ScoutSuite.providers.base.configs import Scout2Config
from ScoutSuite.providers import get_provider
from ScoutSuite.core.ruleset import Ruleset
from ScoutSuite.output.html import RulesetGenerator


########################################
##### Main
########################################

def main():

    # Parse arguments
    parser = RulesArgumentParser()
    args = parser.parse_args()

    # Configure the debug level
    configPrintException(args.debug)

    # Check version of opinel
    if not check_requirements(os.path.realpath(__file__)):
        return 42

    # Load ruleset
    ruleset = Ruleset(filename = args.base_ruleset, name = args.ruleset_name, rules_dir = args.rules_dir, ruleset_generator = True)

    # Generate the HTML generator
    ruleset_generator = RulesetGenerator(args.ruleset_name, args.generator_dir)

    #FIXME this is broken in Scout Suite
    # Create a cloud provider object
    cloud_provider = get_provider(provider='aws',
                                  profile='default')

    ruleset.ruleset_generator_metadata = Scout2Config('default', None, None, [], []).metadata

    ruleset_generator_path = ruleset_generator.save(ruleset, args.force_write, args.debug)

    # Open the HTML ruleset generator in a browser
    if not args.no_browser:
        printInfo('Starting the HTML ruleset generator...')
        url = 'file://%s' % os.path.abspath(ruleset_generator_path)
        webbrowser.open(url, new=2)



# -*- coding: utf-8 -*-

# Python2 vs Python3
try:
    from Queue import Queue
except ImportError:
    from queue import Queue

import json

from googleapiclient.errors import HttpError
from google.api_core.exceptions import PermissionDenied

from opinel.utils.console import printException, printError

from ScoutSuite.providers.base.configs.base import BaseConfig


class GCPBaseConfig(BaseConfig):

    def __init__(self, thread_config=4, projects=[], **kwargs):

        self.projects = projects

        super(GCPBaseConfig, self).__init__(thread_config)

    def _is_provider(self, provider_name):
        if provider_name == 'gcp':
            return True
        else:
            return False

    def get_zones(self, **kwargs):
        """
        Certain services require to be poled per-zone. In these cases, this method will return a list of zones to poll
        or None.

        :return:
        """
        return None

    def _get_method(self, api_client, target_type, list_method_name):
        """
        Gets the appropriate method, required as each provider may have particularities

        :return:
        """

        # This is a specific case for GCP services that don't have a native cloud library
        if self.library_type == 'api_client_library':
            # Required for nested targers, e.g. projects().resource().list()
            target = getattr(api_client, target_type.split('.')[0])
            for t in target_type.split('.')[1:]:
                target = getattr(target(), t)
                # target_type = type  # hack to display the last element
            method = getattr(target(), list_method_name)
        # This works for AWS and GCP cloud libraries
        else:
            method = getattr(api_client, list_method_name)

        return method

    def _get_targets(self, response_attribute, api_client, method, list_params, ignore_list_error):
        """
        Fetch the targets, required as each provider may have particularities

        :return:
        """

        targets = []
        error_list = [] # list of errors, so that we don't print the same error multiple times

        try:

            # FIXME this is temporary, will have to be moved to Config children objects
            # get zones from a project that has CE API enabled
            zones = None
            for project in self.projects:
                try:
                    zones = self.get_zones(client=api_client, project=project['projectId'])
                except HttpError as e:
                    pass
                except Exception as e:
                    printException(e)
                if zones:
                    break
            # What this does is create a list with all combinations of possibilities for method parameters
            list_params_list = []
            # only projects
            if ('project_placeholder' in list_params.values() or 'projects/project_placeholder' in list_params.values())\
                    and not 'zone_placeholder' in list_params.values():
                for project in self.projects:
                    list_params_list.append({key:
                                                 project['projectId'] if list_params[key] == 'project_placeholder'
                                                 else ('projects/%s' % project['projectId'] if list_params[key] == 'projects/project_placeholder'
                                                       else list_params[key])
                                             for key in list_params})
            # only zones
            elif not ('project_placeholder' in list_params.values() or 'projects/project_placeholder' in list_params.values())\
                    and 'zone_placeholder' in list_params.values():
                for zone in zones:
                    list_params_list.append({key:
                                                 zone if list_params[key] == 'zone_placeholder'
                                                 else list_params[key]
                                             for key in list_params})
            # projects and zones
            elif ('project_placeholder' in list_params.values() or 'projects/project_placeholder' in list_params.values())\
                    and 'zone_placeholder' in list_params.values():
                import itertools
                for elem in list(itertools.product(*[self.projects, zones])):
                    list_params_list.append({key:
                                                 elem[0]['projectId'] if list_params[key] == 'project_placeholder'
                                                 else ('projects/%s' % elem[0]['projectId'] if list_params[key] == 'projects/project_placeholder'
                                                       else (elem[1] if list_params[key] == 'zone_placeholder'
                                                             else list_params[key]))
                                             for key in list_params})
            # neither projects nor zones
            else:
                list_params_list.append(list_params)

            for list_params_combination in list_params_list:

                try:

                    if self.library_type == 'cloud_client_library':
                        response = method(**list_params_combination)
                        targets += list(response)
                        # Remove client as it's unpickleable and adding the object to the Queue will pickle
                        # The client is later re-inserted in each Config
                        for t in targets:
                            t._client = None

                    if self.library_type == 'api_client_library':

                        # TODO need to handle long responses
                        request = method(**list_params_combination)
                        while request is not None:
                            response = request.execute()

                            if 'items' in response:
                                targets += response['items']
                            # TODO this should be more modular
                            # this is only for cloudresourcemanager
                            if 'bindings' in response:
                                targets += response['bindings']
                            # this is only for IAM
                            if 'accounts' in response:
                                targets += response['accounts']

                            # TODO need to define the _next to handle long responses
                            # request = method_next(previous_request=request,
                            #                       previous_response=response)
                            request = None

                except HttpError as e:
                    error_json = json.loads(e.content)
                    if error_json['error']['message'] not in error_list:
                        error_list.append(error_json['error']['message'])
                        printError(error_json['error']['message'])

                except PermissionDenied as e:
                    printError("%s: %s - %s for project %s" % (e.message, self.service, self.targets, project['projectId']))

                except Exception as e:
                    printException(e)

        except HttpError as e:
            error_json = json.loads(e.content)
            if error_json['error']['message'] not in error_list:
                error_list.append(error_json['error']['message'])
                printError(error_json['error']['message'])

        except Exception as e:
            printException(e)

        finally:
            return targets



# -*- coding: utf-8 -*-

import os

import google.auth
import googleapiclient
from opinel.utils.console import printError, printException

from ScoutSuite.providers.base.provider import BaseProvider
from ScoutSuite.providers.gcp.configs.services import GCPServicesConfig
from ScoutSuite.providers.gcp.utils import gcp_connect_service




class GCPCredentials():

    def __init__(self, api_client_credentials, cloud_client_credentials):
        self.api_client_credentials = api_client_credentials
        self.cloud_client_credentials = cloud_client_credentials

class GCPProvider(BaseProvider):
    """
    Implements provider for AWS
    """

    def __init__(self, project_id=None, folder_id=None, organization_id=None,
                 report_dir=None, timestamp=None, services=[], skipped_services=[], thread_config=4, **kwargs):

        self.profile = 'gcp-profile'  # TODO this is aws-specific

        self.metadata_path = '%s/metadata.json' % os.path.split(os.path.abspath(__file__))[0]

        self.provider_code = 'gcp'
        self.provider_name = 'Google Cloud Platform'

        self.projects=[]
        self.project_id=project_id
        self.folder_id=folder_id
        self.organization_id=organization_id

        self.services_config = GCPServicesConfig

        super(GCPProvider, self).__init__(report_dir, timestamp, services, skipped_services, thread_config)

    def authenticate(self, key_file=None, user_account=None, service_account=None, **kargs):
        """
        Implement authentication for the GCP provider
        Refer to https://google-auth.readthedocs.io/en/stable/reference/google.auth.html.

        :return:
        """

        if user_account:
            # disable GCP warning about using User Accounts
            import warnings
            warnings.filterwarnings("ignore", "Your application has authenticated using end user credentials")
            pass  # Nothing more to do
        elif service_account:
            client_secrets_path = os.path.abspath(key_file)  # TODO this is probably wrong
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = client_secrets_path
        else:
            printError('Failed to authenticate to GCP - no supported account type')
            return False

        try:

            self.credentials, project_id = google.auth.default()

            if self.credentials:

                if self.project_id:
                    self.projects = self._get_projects(parent_type='project',
                                                       parent_id=self.project_id)
                    self.aws_account_id = self.project_id # FIXME this is for AWS

                elif self.organization_id:
                    self.projects = self._get_projects(parent_type='organization',
                                                                        parent_id=self.organization_id)
                    self.aws_account_id = self.organization_id # FIXME this is for AWS

                elif self.folder_id:
                    self.projects = self._get_projects(parent_type='folder',
                                                                        parent_id=self.folder_id)
                    self.aws_account_id = self.folder_id # FIXME this is for AWS

                else:
                    self.projects = [project_id]

                # TODO this shouldn't be done here? but it has to in order to init with projects...
                self.services.set_projects(projects=self.projects)

                return True
            else:
                return False

        except google.auth.exceptions.DefaultCredentialsError as e:
            printError('Failed to authenticate to GCP')
            printException(e)
            return False

        except googleapiclient.errors.HttpError as e:
            printError('Failed to authenticate to GCP')
            printException(e)
            return False


    def preprocessing(self, ip_ranges=[], ip_ranges_name_key=None):
        """
        TODO description
        Tweak the AWS config to match cross- resources and clean any fetching artifacts

        :param ip_ranges:
        :param ip_ranges_name_key:
        :return: None
        """

        self._match_instances_and_snapshots()
        self._match_networks_and_instances()

        super(GCPProvider, self).preprocessing()

    def _get_projects(self, parent_type, parent_id):
        """
        Returns all the projects in a given organization or folder. For a project_id it only returns the project
        details.
        """

        if parent_type not in ['project', 'organization', 'folder']:
            return None

        projects = []

        #FIXME can't currently be done with API client library as it consumes v1 which doesn't support folders
        """
        
        resource_manager_client = resource_manager.Client(credentials=self.credentials)

        project_list = resource_manager_client.list_projects()

        for p in project_list:
            if p.parent['id'] == self.organization_id and p.status == 'ACTIVE':
                projects.append(p.project_id)
        """

        resource_manager_client_v1 = gcp_connect_service(service='cloudresourcemanager', credentials=self.credentials)
        resource_manager_client_v2 = gcp_connect_service(service='cloudresourcemanager-v2', credentials=self.credentials)

        if parent_type == 'project':

            project_response = resource_manager_client_v1.projects().list(filter='id:%s' % parent_id).execute()
            if 'projects' in project_response.keys():
                for project in project_response['projects']:
                    if project['lifecycleState'] == "ACTIVE":
                        projects.append(project)

        else:

            # get parent children projectss
            request = resource_manager_client_v1.projects().list(filter='parent.id:%s' % parent_id)
            while request is not None:
                response = request.execute()

                if 'projects' in response.keys():
                    for project in response['projects']:
                        if project['lifecycleState'] == "ACTIVE":
                            projects.append(project)

                request = resource_manager_client_v1.projects().list_next(previous_request=request,
                                                                          previous_response=response)

            # get parent children projects in children folders recursively
            folder_response = resource_manager_client_v2.folders().list(parent='%ss/%s' % (parent_type, parent_id)).execute()
            if 'folders' in folder_response.keys():
                for folder in folder_response['folders']:
                    projects.extend(self._get_projects("folder", folder['name'].strip(u'folders/')))

        return projects

    def _match_instances_and_snapshots(self):
        """
        Compare Compute Engine instances and snapshots to identify instance disks that do not have a snapshot.

        :return:
        """

        if 'computeengine' in self.services:
            for instance in self.services['computeengine']['instances'].values():
                for instance_disk in instance['disks'].values():
                    instance_disk['snapshots'] = []
                    for disk in self.services['computeengine']['snapshots'].values():
                        if disk['status'] == 'READY' and disk['source_disk_url'] == instance_disk['source_url']:
                            instance_disk['snapshots'].append(disk)

                    instance_disk['latest_snapshot'] = max(instance_disk['snapshots'],
                                                           key=lambda x: x['creation_timestamp']) \
                        if instance_disk['snapshots'] else None


    def _match_networks_and_instances(self):
        """
        For each network, math instances in that network

        :return:
        """

        if 'computeengine' in self.services:
            for network in self.services['computeengine']['networks'].values():
                network['instances'] = []
                for instance in self.services['computeengine']['instances'].values():
                    for network_interface in instance['network_interfaces']:
                        if network_interface['network'] == network['network_url']:
                            network['instances'].append(instance['id'])



# -*- coding: utf-8 -*-

import os

import google.auth
from opinel.utils.console import printError, printException

from ScoutSuite.providers.base.provider import BaseProvider
from ScoutSuite.providers.azure.configs.services import AzureServicesConfig

from azure.common.credentials import ServicePrincipalCredentials

class AzureCredentials():

    def __init__(self, credentials, subscription_id):
        self.credentials = credentials
        self.subscription_id = subscription_id

class AzureProvider(BaseProvider):
    """
    Implements provider for AWS
    """

    def __init__(self, project_id=None, organization_id=None,
                 report_dir=None, timestamp=None, services=[], skipped_services=[], thread_config=4, **kwargs):

        self.profile = 'aws-profile'  # TODO this is aws-specific

        self.metadata_path = '%s/metadata.json' % os.path.split(os.path.abspath(__file__))[0]

        self.provider_code = 'azure'
        self.provider_name = 'Microsoft Azure'

        self.services_config = AzureServicesConfig

        super(AzureProvider, self).__init__(report_dir, timestamp, services, skipped_services, thread_config)

    def authenticate(self, key_file=None, user_account=None, service_account=None, **kargs):
        """
        Implement authentication for the GCP provider
        Refer to https://google-auth.readthedocs.io/en/stable/reference/google.auth.html.

        :return:
        """

        try:

            # TODO this is temporary
            import os
            # Azure subscription
            SUBSCRIPTION_ID = os.environ['SUBSCRIPTION_ID']
            # Tenant ID for your Azure Subscription
            TENANT_ID = os.environ['TENANT_ID']
            # Your Service Principal App ID
            CLIENT = os.environ['CLIENT']
            # Your Service Principal Password
            KEY = os.environ['KEY']

            self.aws_account_id = TENANT_ID # TODO this is for AWS

            credentials = ServicePrincipalCredentials(
                client_id=CLIENT,
                secret=KEY,
                tenant=TENANT_ID
            )

            self.credentials = AzureCredentials(credentials, SUBSCRIPTION_ID)

            return True

        except Exception as e:
            printError('Failed to authenticate to Azure')
            printException(e)
            return False

    def preprocessing(self, ip_ranges=[], ip_ranges_name_key=None):
        """
        TODO description
        Tweak the AWS config to match cross-service resources and clean any fetching artifacts

        :param ip_ranges:
        :param ip_ranges_name_key:
        :return: None
        """

        super(AzureProvider, self).preprocessing()



# -*- coding: utf-8 -*-

from opinel.utils.console import printError, printException, printDebug

from ScoutSuite.providers.base.configs.services import BaseServicesConfig
from ScoutSuite.providers.azure.services.storageaccounts import StorageAccountsConfig

class AzureServicesConfig(BaseServicesConfig):

    def __init__(self, metadata=None, thread_config=4, **kwargs):

        self.storageaccounts = StorageAccountsConfig(thread_config=thread_config)

    def _is_provider(self, provider_name):
        if provider_name == 'azure':
            return True
        else:
            return False



# -*- coding: utf-8 -*-

__author__  = 'NCC Group'
__version__ = '4.0.0'

AWSCONFIG  = 42
EXCEPTIONS = 4242
HTMLREPORT = 424242
AWSRULESET = 42424242

DEFAULT_REPORT_DIR = 'scoutsuite-report'
AWSCONFIG_FILE  = 'inc-scoutsuite-run/scoutsuite_config.js'
AWSRULESET_FILE = 'inc-scoutsuite-run/scoutsuite_ruleset.js'
EXCEPTIONS_FILE = 'inc-scoutsuite-run/scoutsuite_exceptions.js'
HTMLREPORT_FILE = 'report.html'
GENERATOR_FILE  = 'ruleset-generator.html'

REPORT_TITLE  = 'Scout Suite Report'
